#include "stdafx.h"
#include "CD_RIPPER.h"
#include "CD_RIPPERDlg.h"

#include "basscd.h"
#include "basswma.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


CCD_RIPPERDlg::CCD_RIPPERDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CCD_RIPPERDlg::IDD, pParent)
	, m_strOutputFile(_T(""))
	, m_nAlbumMode(0)
	, m_strProgress(_T(""))
	, m_nResample(false)
	, m_nResampleFreq(0)
	, m_nResampleChan(0)
	, m_nVbr(0)
	, m_nPro(0)
	, m_strKbps(_T(""))
	, m_nTrack(0)
	, m_strEncoderCmdLine(_T("")),
	m_bRipping(false)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CCD_RIPPERDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_OPTICAL_DRIVES, m_opticalDrives);
	DDX_Text(pDX, IDC_OUTPUT_FILE, m_strOutputFile);
	DDX_Text(pDX, IDC_PROGRESS, m_strProgress);
	DDX_Check(pDX, IDC_ALBUM_MODE, m_nAlbumMode);
	DDX_Check(pDX, IDC_RESAMPLE, m_nResample);
	DDX_Text(pDX, IDC_RESAMPLE_FREQ, m_nResampleFreq);
	DDX_Text(pDX, IDC_RESAMPLE_CHAN, m_nResampleChan);
	DDX_Text(pDX, IDC_ENCODER_CMD_LINE, m_strEncoderCmdLine);
	DDX_Control(pDX, IDC_WMA_KBPS, m_wmaKbps);
	DDX_Check(pDX, IDC_VBR, m_nVbr);
	DDX_Check(pDX, IDC_PRO, m_nPro);
	DDX_Text(pDX, IDC_KBPS, m_strKbps);
	DDX_Text(pDX, IDC_TRACK_NO, m_nTrack);
}

BEGIN_MESSAGE_MAP(CCD_RIPPERDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_RIP_WAV, &CCD_RIPPERDlg::OnBnClickedRipWav)
	ON_BN_CLICKED(IDC_STOP_RIP, &CCD_RIPPERDlg::OnBnClickedStopRip)
	ON_BN_CLICKED(IDC_QUERY, &CCD_RIPPERDlg::OnBnClickedQuery)
	ON_BN_CLICKED(IDC_VBR, &CCD_RIPPERDlg::OnBnClickedVbr)
	ON_BN_CLICKED(IDC_PRO, &CCD_RIPPERDlg::OnBnClickedPro)
	ON_BN_CLICKED(IDC_RIP_SINGLE_TRACK, &CCD_RIPPERDlg::OnBnClickedRipSingleTrack)
	ON_BN_CLICKED(IDC_RIP_WMA, &CCD_RIPPERDlg::OnBnClickedRipWma)
	ON_BN_CLICKED(IDC_RIP_EXTERNAL_ENCODER, &CCD_RIPPERDlg::OnBnClickedRipExternalEncoder)
	ON_WM_SETCURSOR()
END_MESSAGE_MAP()

/*****************************************************************************************
* Purpose:		Initialize the dialog.
* Returns:		See MSDN.
*****************************************************************************************/
BOOL CCD_RIPPERDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	m_strOutputFile = _T("C:\\alma.wav");
	m_nResampleChan = 1;
	m_nResampleFreq = 22050;
	m_strEncoderCmdLine = _T("C:\\flac.exe -o \"{o}\" -8 --verify --endian=little --channels={c} --bps=16 --sample-rate={s} --sign=signed -");
	m_strProgress = _T("0% - 0%");
	UpdateData(false);

	if (!BASSCDRipper::InitBASSCDRipperLibrary())
	{
		CString strErrorMsg;
		strErrorMsg.Format(_T("Error loading %s!"), BASSCDRipperDLL);
		AfxMessageBox(strErrorMsg);
		PostQuitMessage(0);
	}
	
	// Initialise BASS
    InitBASS();

	// Fill the drive list
    EnumerateDrives();
    
    // Create a ripper object
    m_pBASSCDRipper = BASSCDRipper::BASSCDRipper_Create();

	return TRUE;
}

/*****************************************************************************************
* Purpose:		Initialize BASS.
*****************************************************************************************/
void CCD_RIPPERDlg::InitBASS()
{
	// check whether the correct BASS version was loaded
	if (BASSVERSION != HIWORD(BASS_GetVersion()))
	{
		AfxMessageBox(_T("An incorrect version of BASS.DLL was loaded"));
		PostQuitMessage(0);
	}

	// Initialize audio - default device, 44100hz, stereo, 16 bits
	if (!BASS_Init(-1, 44100, 0, GetSafeHwnd(), NULL))
	{
		Error(_T("Error initializing audio!"));
	}

	AfxMessageBox(_T("BASS CD Ripper Library uses BASS, BASSCD, BASSEnc, BASSWMA and BASSMix.\r\n\
You need a separate license to use them in your application.\r\n\
Available from: http://www.un4seen.com/"));
}

/*****************************************************************************************
* Purpose:		Populate the "Optical drive" combobox.
*****************************************************************************************/
void CCD_RIPPERDlg::EnumerateDrives()
{
	BASS_CD_INFO cdInfo;
	bool bDriveReturned = true;
	for (int i = 0; (i < MAXDRIVES) && bDriveReturned; i++)
	{
		::ZeroMemory(&cdInfo, sizeof(BASS_CD_INFO));
		bDriveReturned = BASS_CD_GetInfo(i, &cdInfo);
		if(bDriveReturned)
		{
			CString strDriveName;
			strDriveName.Format(_T("%c: %s %s"), cdInfo.letter + 'A', (LPCTSTR)CA2T(cdInfo.vendor), 
																		(LPCTSTR)CA2T(cdInfo.product));
 			m_opticalDrives.AddString(strDriveName);
		}
	}
	m_opticalDrives.SetCurSel(0);
}

/*****************************************************************************************
* Purpose:		Display an error message and append the BASS error code to it.
* Parameters:	szErrorMsg - message do show.
*****************************************************************************************/
void CCD_RIPPERDlg::Error(LPCTSTR szErrorMsg)
{
	CString strErrorMsg;
	strErrorMsg.Format(_T("%s\r\n(Error code: %d)"), szErrorMsg, BASS_ErrorGetCode());
}

/*****************************************************************************************
* Purpose:		Bass CD Ripper progress callback
* Parameters:	BASSCDRipper - pointer to the initialized BASS CD Ripper instance (returned
*			by BASSCDRipper_Create()
*				Progress - the progress of the current track
*				TotalProgress - the overall progress of the ripping
*****************************************************************************************/
void __stdcall CCD_RIPPERDlg::OnProgress(LPVOID BASSCDRipper, UINT Progress, UINT TotalProgress)
{
	CCD_RIPPERDlg *pDlg = ((CCD_RIPPERDlg*)AfxGetMainWnd());
	pDlg->m_strProgress.Format(_T("%d%% - %d%%"), TotalProgress, Progress);
	pDlg->UpdateData(false);
}

/*****************************************************************************************
* Purpose:		WMA OnBefore callback can be used to set meta data
* Parameters:	BASSCDRipper - pointer to the initialized BASS CD Ripper instance (returned
*			by BASSCDRipper_Create()
*				WMAEncodingHandle - 
*				Track - track number
*****************************************************************************************/
void __stdcall CCD_RIPPERDlg::WMAEncBefore(LPVOID BASSCDRipper, UINT WMAEncodingHandle, int Track)
{
#ifdef UNICODE
#define BASS_WMA_TAG_PARAMETER BASS_WMA_TAG_UNICODE
#else
#define BASS_WMA_TAG_PARAMETER BASS_WMA_TAG_ANSI
#endif
	//* See BASSWMA: BASS_WMA_EncodeSetTag() documentation for details
    //* Set Title data
	CString strTitle;
	strTitle.Format(_T("Title%d"), Track);
	BASS_WMA_EncodeSetTag(WMAEncodingHandle, (char*)_T("Title"), (char*)strTitle.GetString(), BASS_WMA_TAG_PARAMETER);
    //* Set URL data
    BASS_WMA_EncodeSetTag(WMAEncodingHandle, (char*)_T("WM/PromotionURL"),(char*) _T("http://www.3delite.hu/"), BASS_WMA_TAG_PARAMETER);
}

/*****************************************************************************************
* Purpose:		Initialize the CD Ripper with the napes of the individual tracks if 
*			"Album mode" is not checked.
* Parameters:	nDrive - 0-based index of optical drive to rip from
*				szDrive - target drive
*				szDirectory - target directory
*				szExtension - file extension, beginning with .
*****************************************************************************************/
void CCD_RIPPERDlg::InitializeTracksIfNeeded(int nDrive, LPCTSTR szDrive, LPCTSTR szDirectory, LPCTSTR szExtension)
{
	if (!m_nAlbumMode)
	{
		// Multiple tracks mode	
        // Fill tracks data
		BASSCDRipper::BASSCDRipper_ClearTracks(m_pBASSCDRipper);
		for (int i = 0; i < BASS_CD_GetTracks(nDrive); i++)
		{
			CString strTrackFileName;
			strTrackFileName.Format(_T("%s%sTrack%d%s"), szDrive, szDirectory, i + 1, szExtension);
			BASSCDRipper::BASSCDRipper_AddTracks(m_pBASSCDRipper, (LPTSTR)strTrackFileName.GetString());
		}
	}
}

/*****************************************************************************************
* Purpose:		Click handler for the "Rip sto wav" button.
*****************************************************************************************/
void CCD_RIPPERDlg::OnBnClickedRipWav()
{
	m_bRipping = true;
	UpdateData();

	int nSelectedDrive = m_opticalDrives.GetCurSel();
	TBASSCDRipperParams Params;
	::ZeroMemory(&Params, sizeof(Params));

	Params.Drive = nSelectedDrive;
	Params.AlbumMode = m_nAlbumMode != 0;
	Params.FileName = (LPTSTR)m_strOutputFile.GetString();
	Params.Format = FORMAT_WAV;
	if(m_nResample)
	{
		Params.Resample = true;
		Params.ResampleFreq = m_nResampleFreq;
		Params.ResampleChannels = m_nResampleChan;
	}

	TCHAR szDrive[_MAX_DRIVE];
	TCHAR szDirectory[_MAX_DIR];
	TCHAR szExtension[_MAX_EXT];
	_tsplitpath(m_strOutputFile, szDrive, szDirectory, NULL, szExtension);
	InitializeTracksIfNeeded(nSelectedDrive, szDrive, szDirectory, szExtension);
	
	BASSCDRipper::BASSCDRipper_SetParams(m_pBASSCDRipper, Params);
	BASSCDRipper::BASSCDRipper_SetProgressCallback(m_pBASSCDRipper, OnProgress);
	BASSCDRipper::BASSCDRipper_Ripp(m_pBASSCDRipper);
	
	m_bRipping = false;
	AfxMessageBox(_T("Completed"));
}


/*****************************************************************************************
* Purpose:		Click handler for the "Rip ingle track" button.
*****************************************************************************************/
void CCD_RIPPERDlg::OnBnClickedRipSingleTrack()
{
	m_bRipping = true;

	UpdateData();

	int nSelectedDrive = m_opticalDrives.GetCurSel();

	// Ripp single track in WAV mode
	TBASSCDRipperParams Params;
	::ZeroMemory(&Params, sizeof(Params));
	Params.Drive = nSelectedDrive;
	Params.FileName = (LPTSTR)m_strOutputFile.GetString();
	Params.Format = FORMAT_WAV;
	if(m_nResample)
	{
		Params.Resample = true;
		Params.ResampleFreq = m_nResampleFreq;
		Params.ResampleChannels = m_nResampleChan;
	}
	BASSCDRipper::BASSCDRipper_SetParams(m_pBASSCDRipper, Params);
	BASSCDRipper::BASSCDRipper_SetProgressCallback(m_pBASSCDRipper, OnProgress);
	BASSCDRipper::BASSCDRipper_RippTrack(m_pBASSCDRipper, m_nTrack, (LPTSTR)m_strOutputFile.GetString());
	
	 // Ripp single track with encoder
	/*TBASSCDRipperParams Params;
	::ZeroMemory(&Params, sizeof(Params));
	Params.Drive = nSelectedDrive;
	Params.FileName = (LPTSTR)m_strOutputFile.GetString();
	Params.Format = FORMAT_ENC;
	// {o} - will be changed to output file name
	Params.EncCmdLine = (LPTSTR)m_strEncoderCmdLine.GetString();
	if(m_nResample)
	{
		Params.Resample = true;
		Params.ResampleFreq = m_nResampleFreq;
		Params.ResampleChannels = m_nResampleChan;
	}
	BASSCDRipper::BASSCDRipper_SetParams(m_pBASSCDRipper, Params);
	BASSCDRipper::BASSCDRipper_SetProgressCallback(m_pBASSCDRipper, OnProgress);
	BASSCDRipper::BASSCDRipper_RippTrack(m_pBASSCDRipper, m_nTrack, (LPTSTR)m_strOutputFile.GetString());
	*/

	// Ripp single track in WMA mode
	/*TBASSCDRipperParams Params;
	::ZeroMemory(&Params, sizeof(Params));
	Params.Drive = nSelectedDrive;
	Params.FileName = (LPTSTR)m_strOutputFile.GetString();
	Params.Format = FORMAT_WMA;
	CString strSelectedKbps;
	m_wmaKbps.GetLBText(m_wmaKbps.GetCurSel(), strSelectedKbps);
	Params.WMAQuality = _ttoi(strSelectedKbps.GetString());
	if (!m_nVbr)
	{
		Params.WMAQuality *= 1000;
	}

	if (m_nPro)
	{
		Params.WMAEncFlags = BASS_WMA_ENCODE_PRO;
	}
	if(m_nResample)
	{
		Params.Resample = true;
		Params.ResampleFreq = m_nResampleFreq;
		Params.ResampleChannels = m_nResampleChan;
	}
	BASSCDRipper::BASSCDRipper_SetParams(m_pBASSCDRipper, Params);
	BASSCDRipper::BASSCDRipper_SetProgressCallback(m_pBASSCDRipper, OnProgress);
	BASSCDRipper::BASSCDRipper_RippTrack(m_pBASSCDRipper, m_nTrack, (LPTSTR)m_strOutputFile.GetString());
	*/

	m_bRipping = false;
	AfxMessageBox(_T("Completed"));
}

/*****************************************************************************************
* Purpose:		Click handler for the "Rip to WMA" button.
*****************************************************************************************/
void CCD_RIPPERDlg::OnBnClickedRipWma()
{
	m_bRipping = true;

	UpdateData();

	int nSelectedDrive = m_opticalDrives.GetCurSel();
	TBASSCDRipperParams Params;
	::ZeroMemory(&Params, sizeof(Params));

	Params.Drive = nSelectedDrive;
	Params.AlbumMode = m_nAlbumMode != 0;

	TCHAR szDrive[_MAX_DRIVE];
	TCHAR szDirectory[_MAX_DIR];
	TCHAR szFName[_MAX_FNAME];
	_tsplitpath(m_strOutputFile, szDrive, szDirectory, szFName, NULL);
	m_strOutputFile.Format(_T("%s%s%s.wma"), szDrive, szDirectory, szFName);

	Params.FileName = (LPTSTR)m_strOutputFile.GetString();
	Params.Format = FORMAT_WMA;
	if(m_nResample)
	{
		Params.Resample = true;
		Params.ResampleFreq = m_nResampleFreq;
		Params.ResampleChannels = m_nResampleChan;
	}

	CString strSelectedKbps;
	m_wmaKbps.GetLBText(m_wmaKbps.GetCurSel(), strSelectedKbps);
	Params.WMAQuality = _ttoi(strSelectedKbps.GetString());
	if (!m_nVbr)
	{
		Params.WMAQuality *= 1000;
	}

	if (m_nPro)
	{
		Params.WMAEncFlags = BASS_WMA_ENCODE_PRO;
	}

	InitializeTracksIfNeeded(nSelectedDrive, szDrive, szDirectory, _T(".wma"));
	
	BASSCDRipper::BASSCDRipper_SetParams(m_pBASSCDRipper, Params);
	BASSCDRipper::BASSCDRipper_SetProgressCallback(m_pBASSCDRipper, OnProgress);
	BASSCDRipper::BASSCDRipper_SetWMAEncCallback(m_pBASSCDRipper, WMAEncBefore);
	BASSCDRipper::BASSCDRipper_Ripp(m_pBASSCDRipper);
	
	m_bRipping = false;
	AfxMessageBox(_T("Completed"));
}

/*****************************************************************************************
* Purpose:		Click handler for the "Rip with external encoder" button.
*****************************************************************************************/
void CCD_RIPPERDlg::OnBnClickedRipExternalEncoder()
{
	m_bRipping = true;

	UpdateData();

	int nSelectedDrive = m_opticalDrives.GetCurSel();
	TBASSCDRipperParams Params;
	::ZeroMemory(&Params, sizeof(Params));

	Params.Drive = nSelectedDrive;
	Params.AlbumMode = m_nAlbumMode != 0;

	TCHAR szDrive[_MAX_DRIVE];
	TCHAR szDirectory[_MAX_DIR];
	TCHAR szFName[_MAX_FNAME];
	_tsplitpath(m_strOutputFile, szDrive, szDirectory, szFName, NULL);
	m_strOutputFile.Format(_T("%s%s%s.flac"), szDrive, szDirectory, szFName);

	Params.FileName = (LPTSTR)m_strOutputFile.GetString();
	Params.Format = FORMAT_ENC;
	if(m_nResample)
	{
		Params.Resample = true;
		Params.ResampleFreq = m_nResampleFreq;
		Params.ResampleChannels = m_nResampleChan;
	}

	// {o} - will be changed to output file name
	Params.EncCmdLine = (LPTSTR)m_strEncoderCmdLine.GetString();
	Params.EncoderPriority = NORMAL_PRIORITY_CLASS;
	
	InitializeTracksIfNeeded(nSelectedDrive, szDrive, szDirectory, _T(".flac"));
	
	BASSCDRipper::BASSCDRipper_SetParams(m_pBASSCDRipper, Params);
	BASSCDRipper::BASSCDRipper_SetProgressCallback(m_pBASSCDRipper, OnProgress);
	BASSCDRipper::BASSCDRipper_Ripp(m_pBASSCDRipper);
	
	m_bRipping = false;
	AfxMessageBox(_T("Completed"));
}

/*****************************************************************************************
* Purpose:		Click handler for the "Stop rip" button.
*****************************************************************************************/
void CCD_RIPPERDlg::OnBnClickedStopRip()
{
	BASSCDRipper::BASSCDRipper_StopRipp(m_pBASSCDRipper);
}

/*****************************************************************************************
* Purpose:		Click handler for the "Query" button.
*****************************************************************************************/
void CCD_RIPPERDlg::OnBnClickedQuery()
{
	UpdateData();

	UINT Flags;
	int Freq, Channels;

	m_wmaKbps.ResetContent();
	if (m_nVbr)
	{
		Flags = BASS_WMA_ENCODE_RATES_VBR;
		m_strKbps = _T("%");
	}
	else
	{
		Flags = 0;
		m_strKbps = _T("kbps");
	}

	UpdateData(false);

	if (m_nResample)
	{
		Freq = m_nResampleFreq;
		Channels = m_nResampleChan;
	}
	else
	{
		Freq = 44100;
		Channels = 2;
	}

	if (m_nPro)
	{
		Flags |= BASS_WMA_ENCODE_PRO;
	}

	const DWORD *Rates = BASS_WMA_EncodeGetRates(Freq, Channels, Flags);
	if (NULL == Rates)
	{
		AfxMessageBox(_T("Can't find codec or unsupported parameters!"));
		m_wmaKbps.AddString(_T("Error"));
		m_wmaKbps.EnableWindow(FALSE);
	}
	else
	{
		m_wmaKbps.EnableWindow();
		m_wmaKbps.AddString(_T("Select..."));
		for (; 0 != *Rates; Rates++)
		{
			CString strRate;
			if ((Flags & BASS_WMA_ENCODE_RATES_VBR) > 0)
			{
				strRate.Format(_T("%d"), *Rates);
			}
			else
			{
				strRate.Format(_T("%d"), *Rates / 1000);
			}
			m_wmaKbps.InsertString(-1, strRate);
		}
	}
	m_wmaKbps.SetCurSel(0);
}

/*****************************************************************************************
* Purpose:		Click handler for the "Vbr" check box.
*****************************************************************************************/
void CCD_RIPPERDlg::OnBnClickedVbr()
{
	OnBnClickedQuery();
}

/*****************************************************************************************
* Purpose:		Click handler for the "Pro" check box.
*****************************************************************************************/
void CCD_RIPPERDlg::OnBnClickedPro()
{
	OnBnClickedQuery();
}

/*****************************************************************************************
* Purpose:		Set the dialog's mouse cursor according to the application's state.
* Parameters:	See MSDN
* Returns:		See MSDN
*****************************************************************************************/
BOOL CCD_RIPPERDlg::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message)
{
	SetCursor(AfxGetApp()->LoadStandardCursor(m_bRipping ? IDC_APPSTARTING : IDC_ARROW));

	return true;
}