#include "StdAfx.h"
#include "BASSCDRipperDefs.h"


namespace BASSCDRipper
{
	static HMODULE BASSCDRipperDLLHandle = NULL;
	bool BASSCDRipperDLLLoaded = false;

	t_BASSCDRipper_Create BASSCDRipper_Create = NULL;
	t_BASSCDRipper_Free BASSCDRipper_Free = NULL;
	t_BASSCDRipper_Ripp BASSCDRipper_Ripp = NULL;
	t_BASSCDRipper_StopRipp BASSCDRipper_StopRipp = NULL;
	t_BASSCDRipper_GetLastError BASSCDRipper_GetLastError = NULL;
	t_BASSCDRipper_SetProgressCallback BASSCDRipper_SetProgressCallback = NULL;
	t_BASSCDRipper_SetWMAEncCallback BASSCDRipper_SetWMAEncCallback = NULL;
	t_BASSCDRipper_SetGetChannelCallback BASSCDRipper_SetGetChannelCallback = NULL;
	t_BASSCDRipper_SetCDDataProcCallback BASSCDRipper_SetCDDataProcCallback = NULL;
	t_BASSCDRipper_SetParams BASSCDRipper_SetParams = NULL;
	t_BASSCDRipper_GetParams BASSCDRipper_GetParams = NULL;
	t_BASSCDRipper_ClearTracks BASSCDRipper_ClearTracks = NULL;
	t_BASSCDRipper_AddTracks BASSCDRipper_AddTracks = NULL;
	t_BASSCDRipper_RippTrack BASSCDRipper_RippTrack = NULL;

	/*****************************************************************************************
	* Purpose:		Initialize the BASS CD Ripper library.
	* Returns:		true on success, false otherwise.
	*****************************************************************************************/
	bool InitBASSCDRipperLibrary()
	{
		BASSCDRipperDLLHandle = ::LoadLibrary(BASSCDRipperDLL);
		if(NULL != BASSCDRipperDLLHandle)
		{
			BASSCDRipper_Create = (t_BASSCDRipper_Create)::GetProcAddress(BASSCDRipperDLLHandle, NAME_BASSCDRipperCreate);
			BASSCDRipper_Free = (t_BASSCDRipper_Free)::GetProcAddress(BASSCDRipperDLLHandle, NAME_BASSCDRipperFree);
			BASSCDRipper_Ripp = (t_BASSCDRipper_Ripp)::GetProcAddress(BASSCDRipperDLLHandle, NAME_BASSCDRipperRipp);
			BASSCDRipper_StopRipp = (t_BASSCDRipper_StopRipp)::GetProcAddress(BASSCDRipperDLLHandle, NAME_BASSCDRipperStopRipp);
			BASSCDRipper_GetLastError = (t_BASSCDRipper_GetLastError)::GetProcAddress(BASSCDRipperDLLHandle, NAME_BASSCDRipperGetLastError);
			BASSCDRipper_SetProgressCallback = (t_BASSCDRipper_SetProgressCallback)::GetProcAddress(BASSCDRipperDLLHandle, NAME_BASSCDRipperSetProgressCallback);
			BASSCDRipper_SetWMAEncCallback = (t_BASSCDRipper_SetWMAEncCallback)::GetProcAddress(BASSCDRipperDLLHandle, NAME_BASSCDRipperSetWMAEncCallback);
			BASSCDRipper_SetGetChannelCallback = (t_BASSCDRipper_SetGetChannelCallback)::GetProcAddress(BASSCDRipperDLLHandle, NAME_BASSCDRipperSetGetChannelCallback);
			BASSCDRipper_SetCDDataProcCallback  = (t_BASSCDRipper_SetCDDataProcCallback)::GetProcAddress(BASSCDRipperDLLHandle, NAME_BASSCDRipperSetCDDataProcCallback);
			BASSCDRipper_SetParams = (t_BASSCDRipper_SetParams)::GetProcAddress(BASSCDRipperDLLHandle, NAME_BASSCDRipperSetParams);
			BASSCDRipper_GetParams = (t_BASSCDRipper_GetParams)::GetProcAddress(BASSCDRipperDLLHandle, NAME_BASSCDRipperGetParams);
			BASSCDRipper_ClearTracks = (t_BASSCDRipper_ClearTracks)::GetProcAddress(BASSCDRipperDLLHandle, NAME_BASSCDRipperClearTracks);
			BASSCDRipper_AddTracks = (t_BASSCDRipper_AddTracks)::GetProcAddress(BASSCDRipperDLLHandle, NAME_BASSCDRipperAddTracks);
			BASSCDRipper_RippTrack = (t_BASSCDRipper_RippTrack)::GetProcAddress(BASSCDRipperDLLHandle, NAME_BASSCDRipperRippTrack);

			if((NULL == BASSCDRipper_Create)
				|| (NULL == BASSCDRipper_Free)
				|| (NULL == BASSCDRipper_Ripp)
				|| (NULL == BASSCDRipper_StopRipp)
				|| (NULL == BASSCDRipper_GetLastError)
				|| (NULL == BASSCDRipper_SetProgressCallback)
				|| (NULL == BASSCDRipper_SetWMAEncCallback)
				|| (NULL == BASSCDRipper_SetGetChannelCallback)
				|| (NULL == BASSCDRipper_SetCDDataProcCallback)
				|| (NULL == BASSCDRipper_SetParams)
				|| (NULL == BASSCDRipper_GetParams)
				|| (NULL == BASSCDRipper_ClearTracks)
				|| (NULL == BASSCDRipper_AddTracks)
				|| (NULL == BASSCDRipper_RippTrack))
			{
				BASSCDRipperDLLLoaded = false;
			}
			else
			{
				BASSCDRipperDLLLoaded = true;
			}
		}

		return BASSCDRipperDLLLoaded;
	}

	/*****************************************************************************************
	* Purpose:		Free BASS CD Ripper library.
	* Returns:		true on success, false otherwise.
	*****************************************************************************************/
	bool FreeBASSCDRipperLibrary()
	{
		if (NULL != BASSCDRipperDLLHandle)
		{
			BASSCDRipperDLLLoaded = !::FreeLibrary(BASSCDRipperDLLHandle);
		}
		return !BASSCDRipperDLLLoaded;
	}
}