//********************************************************************************************************************************
//*                                                                                                                              *
//*     BASS CD Ripper Library 1.1.8.62  3delite 2008-2013                                                                      *
//*     See BASS CD Ripper Library Readme.txt for details                                                                        *
//*                                                                                                                              *
//* Two licenses are available if you use this library for commercial purposes:                                                  *
//* Shareware License: 50 Euros                                                                                                  *
//* Commercial License: 250 Euros                                                                                                *
//*                                                                                                                              *
//*     http://www.shareit.com/product.html?productid=300279207                                                                  *
//*                                                                                                                              *
//*     http://www.3delite.hu/Object%20Pascal%20Developer%20Resources/basscdripperlibrary.html                                   *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit BASSCDRipperDefs;

interface

{$IFDEF MSWINDOWS}
uses
    Windows;
{$ENDIF}

const
    NO_CD_CHANNEL               = 100001;
    ERROR_WRITING               = 100002;
    ERROR_CDSTREAM              = 100003;
    ERROR_STARTINGENCODINGREC   = 100004;

    FORMAT_WAV      = 1;
    FORMAT_ENC      = 2;
    FORMAT_WMA      = 3;

const
    NAME_BASSCDRipperCreate                 = 'BASSCDRipper_Create';
    NAME_BASSCDRipperFree                   = 'BASSCDRipper_Free';
    NAME_BASSCDRipperRipp                   = 'BASSCDRipper_Ripp';
    NAME_BASSCDRipperStopRipp               = 'BASSCDRipper_StopRipp';
    NAME_BASSCDRipperGetLastError           = 'BASSCDRipper_GetLastError';
    NAME_BASSCDRipperSetProgressCallback    = 'BASSCDRipper_SetProgressCallback';
    NAME_BASSCDRipperSetWMAEncCallback      = 'BASSCDRipper_SetWMAEncCallback';
    NAME_BASSCDRipperSetGetChannelCallback  = 'BASSCDRipper_SetGetChannelCallback';
    NAME_BASSCDRipperSetCDDataProcCallback  = 'BASSCDRipper_SetCDDataProcCallback';
    NAME_BASSCDRipperSetParams              = 'BASSCDRipper_SetParams';
    NAME_BASSCDRipperGetParams              = 'BASSCDRipper_GetParams';
    NAME_BASSCDRipperClearTracks            = 'BASSCDRipper_ClearTracks';
    NAME_BASSCDRipperAddTracks              = 'BASSCDRipper_AddTracks';
    NAME_BASSCDRipperRippTrack              = 'BASSCDRipper_RippTrack';

const
    BASSCDRipperDLL = 'BASSCDRipper.dll';

type
    HBASSCDRipper = Pointer;

type
    t_OnProgress    = procedure(BASSCDRipper: HBASSCDRipper; Progress, TotalProgress: Cardinal); stdcall;
    t_WMAEncBefore  = procedure(BASSCDRipper: HBASSCDRipper; WMAEncodingHandle: Cardinal; Track: Integer); stdcall;
    t_GetChannel    = procedure(BASSCDRipper: HBASSCDRipper; ChannelHandle: Cardinal; Track: Integer); stdcall;

type
    TBASSCDRipperParams = record
        Drive: Cardinal;
        AlbumMode: Bool;
        FileName: PChar;
        Format: Cardinal;
        EncCmdLine: PChar;
        NoWAVHeader2Encoder: Boolean;
        EncoderPriority: Cardinal;
        Resample: Bool;
        ResampleFreq: DWORD;
        ResampleChannels: DWORD;
        ResampleFlags: DWORD;
        WMAEncFlags: Cardinal;
        WMAQuality: Integer;
    end;

type
    t_BASSCDRipper_Create                   = function: HBASSCDRipper; stdcall;
    t_BASSCDRipper_Free                     = function (BASSCDRipper: HBASSCDRipper): Bool; stdcall;
    t_BASSCDRipper_Ripp                     = function (BASSCDRipper: HBASSCDRipper): Bool; stdcall;
    t_BASSCDRipper_StopRipp                 = function (BASSCDRipper: HBASSCDRipper): Bool; stdcall;
    t_BASSCDRipper_GetLastError             = function (BASSCDRipper: HBASSCDRipper): Integer; stdcall;
    t_BASSCDRipper_SetProgressCallback      = function (BASSCDRipper: HBASSCDRipper; Callback: Pointer): Bool; stdcall;
    t_BASSCDRipper_SetWMAEncCallback        = function (BASSCDRipper: HBASSCDRipper; Callback: Pointer): Bool; stdcall;
    t_BASSCDRipper_SetGetChannelCallback    = function (BASSCDRipper: HBASSCDRipper; Callback: Pointer): Bool; stdcall;
    t_BASSCDRipper_SetCDDataProcCallback    = function (BASSCDRipper: HBASSCDRipper; Callback: Pointer; User: Pointer): Bool; stdcall;
    t_BASSCDRipper_SetParams                = function (BASSCDRipper: HBASSCDRipper; Params: TBASSCDRipperParams): Bool; stdcall;
    t_BASSCDRipper_GetParams                = function (BASSCDRipper: HBASSCDRipper; var Params: TBASSCDRipperParams): Bool; stdcall;
    t_BASSCDRipper_ClearTracks              = function (BASSCDRipper: HBASSCDRipper): Bool; stdcall;
    t_BASSCDRipper_AddTracks                = function (BASSCDRipper: HBASSCDRipper; TrackSaveFileName: PChar): Bool; stdcall;
    t_BASSCDRipper_RippTrack                = function (BASSCDRipper: HBASSCDRipper; Track: Integer; TrackSaveFileName: PChar): Bool; stdcall;

    function InitBASSCDRipperLibrary: Boolean;
    function FreeBASSCDRipperLibrary: Boolean;

var
    BASSCDRipper_Create: t_BASSCDRipper_Create;
    BASSCDRipper_Free: t_BASSCDRipper_Free;
    BASSCDRipper_Ripp: t_BASSCDRipper_Ripp;
    BASSCDRipper_StopRipp: t_BASSCDRipper_StopRipp;
    BASSCDRipper_GetLastError: t_BASSCDRipper_GetLastError;
    BASSCDRipper_SetProgressCallback: t_BASSCDRipper_SetProgressCallback;
    BASSCDRipper_SetWMAEncCallback: t_BASSCDRipper_SetWMAEncCallback;
    BASSCDRipper_SetGetChannelCallback: t_BASSCDRipper_SetGetChannelCallback;
    BASSCDRipper_SetCDDataProcCallback: t_BASSCDRipper_SetCDDataProcCallback;
    BASSCDRipper_SetParams: t_BASSCDRipper_SetParams;
    BASSCDRipper_GetParams: t_BASSCDRipper_GetParams;
    BASSCDRipper_ClearTracks: t_BASSCDRipper_ClearTracks;
    BASSCDRipper_AddTracks: t_BASSCDRipper_AddTracks;
    BASSCDRipper_RippTrack: t_BASSCDRipper_RippTrack;

var
    BASSCDRipperDLLHandle: THandle = 0;
    BASSCDRipperDLLLoaded: Boolean = False;

implementation

Uses
    SysUtils;

function InitBASSCDRipperLibrary: Boolean;
begin
    BASSCDRipperDLLHandle := SafeLoadLibrary(PChar(BASSCDRipperDLL));
    Result := BASSCDRipperDLLHandle <> 0;
    if Result then begin
        BASSCDRipper_Create             := GetProcAddress(BASSCDRipperDLLHandle, PChar(NAME_BASSCDRipperCreate));
        BASSCDRipper_Free               := GetProcAddress(BASSCDRipperDLLHandle, PChar(NAME_BASSCDRipperFree));
        BASSCDRipper_Ripp               := GetProcAddress(BASSCDRipperDLLHandle, PChar(NAME_BASSCDRipperRipp));
        BASSCDRipper_StopRipp           := GetProcAddress(BASSCDRipperDLLHandle, PChar(NAME_BASSCDRipperStopRipp));
        BASSCDRipper_GetLastError       := GetProcAddress(BASSCDRipperDLLHandle, PChar(NAME_BASSCDRipperGetLastError));
        BASSCDRipper_SetProgressCallback:= GetProcAddress(BASSCDRipperDLLHandle, PChar(NAME_BASSCDRipperSetProgressCallback));
        BASSCDRipper_SetWMAEncCallback  := GetProcAddress(BASSCDRipperDLLHandle, PChar(NAME_BASSCDRipperSetWMAEncCallback));
        BASSCDRipper_SetGetChannelCallback := GetProcAddress(BASSCDRipperDLLHandle, PChar(NAME_BASSCDRipperSetGetChannelCallback));
        BASSCDRipper_SetCDDataProcCallback := GetProcAddress(BASSCDRipperDLLHandle, PChar(NAME_BASSCDRipperSetCDDataProcCallback));
        BASSCDRipper_SetParams          := GetProcAddress(BASSCDRipperDLLHandle, PChar(NAME_BASSCDRipperSetParams));
        BASSCDRipper_GetParams          := GetProcAddress(BASSCDRipperDLLHandle, PChar(NAME_BASSCDRipperGetParams));
        BASSCDRipper_ClearTracks        := GetProcAddress(BASSCDRipperDLLHandle, PChar(NAME_BASSCDRipperClearTracks));
        BASSCDRipper_AddTracks          := GetProcAddress(BASSCDRipperDLLHandle, PChar(NAME_BASSCDRipperAddTracks));
        BASSCDRipper_RippTrack          := GetProcAddress(BASSCDRipperDLLHandle, PChar(NAME_BASSCDRipperRippTrack));
    end;
    if (@BASSCDRipper_Create = nil)
    OR (@BASSCDRipper_Free = nil)
    OR (@BASSCDRipper_Ripp = nil)
    OR (@BASSCDRipper_StopRipp = nil)
    OR (@BASSCDRipper_GetLastError = nil)
    OR (@BASSCDRipper_SetProgressCallback = nil)
    OR (@BASSCDRipper_SetWMAEncCallback = nil)
    OR (@BASSCDRipper_SetGetChannelCallback = nil)
    OR (@BASSCDRipper_SetCDDataProcCallback = nil)
    OR (@BASSCDRipper_SetParams = nil)
    OR (@BASSCDRipper_GetParams = nil)
    OR (@BASSCDRipper_ClearTracks = nil)
    OR (@BASSCDRipper_AddTracks = nil)
    OR (@BASSCDRipper_RippTrack = nil)
        then Result := False;

    if Result
        then BASSCDRipperDLLLoaded := True;
end;

function FreeBASSCDRipperLibrary: Boolean;
begin
    Result := False;
    if BASSCDRipperDLLHandle <> 0 then begin
        Result := FreeLibrary(BASSCDRipperDLLHandle);
        if Result then begin
            BASSCDRipperDLLHandle := 0;
            BASSCDRipperDLLLoaded := False;
        end;
    end;
end;

Initialization

    InitBASSCDRipperLibrary;

Finalization

    FreeBASSCDRipperLibrary;

end.
